-- Most of the comments to the code (and the "well written" part of it :-) are by Phanx, whom I thanks for the helps and patience in explaining. 
-- The layout of the input frame was inspired by the one of : http://www.wowinterface.com/downloads/info21884-AutoRun_Reminder.html
-- The scroll frame code is suggested by Lombra.

local ADDON = ...

-- some other defaults
REMGANK_ENABLE = REMGANK_ENABLE or 1

-- Player Lists
RemGankDB = RemGankDB or {}
RemGankLS = RemGankLS or {}  -- defined also below to initialize. here only to know.
-- RemGankCFG = RemGankCFG or {}

-- ScrollFrame Defaults
local NUM_BUTTONS = 9
local BUTTON_HEIGHT = 20
local BUTTON_WIDTH = 96
local buttons = {}


-- Other defaults & upvalues
local cmd, name, note
local default_note="generic ganker"
local prgname = "|cffffd200remgank|r"
local testganker = "TesTgAnkEr"

local bgfile = "Interface\\ChatFrame\\ChatFrameBackground"
local edgefile = "Interface\\Tooltips\\UI-Tooltip-Border"
local framefontn, framefonth, framefontf = GameFontNormal:GetFont()

-- For general use, I wouldn't recommend obsessively upvaluing every
-- global but it is worth doing for situations like CLEU or OnUpdate
-- where you are accessing them extremely often.
local string_format = string.format
local bit_band = bit.band
local COMBATLOG_OBJECT_CONTROL_PLAYER = COMBATLOG_OBJECT_CONTROL_PLAYER
local date = date

-- GUIDs are not available when the main chunk executes, so you have to
-- wait and fill this in once the combat log starts running.
local playerGUID

-- Define all of the locals (functions) first, and then set their values in whatever order
local RemGank_Check_Player, RemGank_Add_Player, RemGank_Record_Player, RemGank_ListFrame_Update, RemGank_DB_Dump

-- and the frames ...
local RemGank_Input, RemGank_Input_Title, RemGank_Input_Text1
local RemGank_Input_Player, RemGank_Input_Text2, RemGank_Input_Text3
local RemGank_Input_eb1, RemGank_Input_Save_Button, RemGank_Input_Canc_Button

-- event frames
local frame_al, frame_cleu, frame_umu

-- config stuff
local options, title, subtext, enable, titlelist, scrollFrame
local RemGank_Del_Player_Button, RemGank_Input_eb2, RemGank_Add_Player_Button
local dbtoolstitle, RemGank_Rld_Player_Button
local info_name, info_desc, info_kills, info_date, info_loc, info_kills_title, info_loc_title
-- Let's begin ...

-- This finalize the store of the name in the array.
function RemGank_Record_Player(name, note)	

	if note == nil or note == "" then  note = default_note end
	
	-- Define the new row in the array
	RemGankDB[name] = RemGankDB[name] or {} 
	
	-- To prevent the math err on the sum of ( nil + 1 ) below
	RemGankDB[name]["nrkill"] = RemGankDB[name]["nrkill"] or 0

	RemGankDB[name]["name"] = name:lower()
	RemGankDB[name]["desc"] = string.sub(note,1,25) 
	RemGankDB[name]["lastloc"] = GetZoneText() .. "/" .. GetSubZoneText()
	RemGankDB[name]["date"] =  date("%d.%m.%Y %H:%M:%S")
	RemGankDB[name]["nrkill"] = ( RemGankDB[name]["nrkill"] + 1 )	
	
	print(string_format("%s: adding %s [ %s ] [ kills: %i ] ", prgname, name, note, RemGankDB[name]["nrkill"]))	
	
	RemGank_DB_Dump()
	RemGank_ListFrame_Update(scrollFrame)

	
end

-- This show up the PopUp
function RemGank_Add_Player(name) 
	RemGank_Input_Player:SetText(name)		
	RemGankDB[name] = RemGankDB[name] or {} 
	RemGankDB[name]["nrkill"] = RemGankDB[name]["nrkill"] or 0	
	RemGank_Input_Text2:SetText(string_format("You have been killed %i times", ( RemGankDB[name]["nrkill"] + 1 )))
	RemGank_Input:Show()
end

-- Various Checks.
function RemGank_Check_Player(name)
	-- is this a new player ? I want a PopUP here to pass a desc
	if RemGankDB[name] == nil then 		
		print(string_format("%s: [%s] new ganker !!", prgname, name))
		RemGank_Add_Player(name) 
	else -- update only record.
		print(string_format("%s: [%s] already present. skip popup, updating record", prgname, name))
		RemGank_Record_Player(name)
	end 
end 

-- Function to dump DB to the listnames
function RemGank_DB_Dump()
	RemGankLS = {}
	for name in pairs(RemGankDB) do		
		name = name:lower()
		if RemGankDB[name]["name"] then 
			table.insert(RemGankLS, RemGankDB[name]["name"])
		end
	end
	-- Sort the list alphabetically
	table.sort(RemGankLS)	
end

-- ScrollFrame 
function RemGank_ListFrame_Update(self)
	local numItems = #RemGankLS
	FauxScrollFrame_Update(self, numItems, NUM_BUTTONS, BUTTON_HEIGHT)
	local offset = FauxScrollFrame_GetOffset(self)
	for line = 1, NUM_BUTTONS do
		local lineplusoffset = line + offset
		local button = buttons[line]
		if lineplusoffset <= numItems then
			button:SetText(RemGankLS[lineplusoffset])
			button:Show()
		else
			button:Hide()
		end
	end
end

-- Definitions of frames -------------------------------------------------------
RemGank_Input = CreateFrame("Frame","RemGank_Input",UIParent)
RemGank_Input:SetBackdrop({bgFile = bgfile, edgeFile = edgefile, edgeSize = 18, insets = {left = 5, right = 5, top = 5, bottom = 5}})
RemGank_Input:SetBackdropColor(0,0,0,0.8)
RemGank_Input:SetWidth(250)
RemGank_Input:SetHeight(180)
RemGank_Input:SetPoint("CENTER",UIParent)
RemGank_Input:EnableMouse(true)
RemGank_Input:SetMovable(true)
RemGank_Input:RegisterForDrag("LeftButton")
RemGank_Input:SetScript("OnDragStart", function(self) self:StartMoving() end)
RemGank_Input:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
RemGank_Input:Hide()
RemGank_Input:SetClampedToScreen(true)

-- Title
RemGank_Input_Title = RemGank_Input:CreateFontString("RemGank_Input_Title")
RemGank_Input_Title:SetPoint("TOPLEFT", RemGank_Input,"TOPLEFT",85, -15)
RemGank_Input_Title:SetFont(GameFontNormal:GetFont(),(framefonth+2))
RemGank_Input_Title:SetText(prgname)

-- Text Register this ...
RemGank_Input_Text1 = RemGank_Input:CreateFontString("RemGank_Input_Text1")
RemGank_Input_Text1:SetPoint("TOPLEFT", RemGank_Input,"TOPLEFT",20, -45)
RemGank_Input_Text1:SetFont(framefontn,framefonth)
RemGank_Input_Text1:SetTextColor(0.6,0.6,0.6,1)
RemGank_Input_Text1:SetText('Register this kill by')

-- Player Name	
RemGank_Input_Player = RemGank_Input:CreateFontString("RemGank_Input_Player")
RemGank_Input_Player:SetPoint("TOPLEFT", RemGank_Input,"TOPLEFT",20, -60)
RemGank_Input_Player:SetFont(framefontn,(framefonth+2))
RemGank_Input_Player:SetTextColor(1,0,0,1)

-- Text NrKill
RemGank_Input_Text2 = RemGank_Input:CreateFontString("RemGank_Input_Text2")
RemGank_Input_Text2:SetPoint("TOPLEFT", RemGank_Input,"TOPLEFT",20, -83)
RemGank_Input_Text2:SetFont(framefontn,framefonth)
RemGank_Input_Text2:SetTextColor(0.6,0.6,0.6,1)

-- Text Note
RemGank_Input_Text3 = RemGank_Input:CreateFontString("RemGank_Input_Text3")
RemGank_Input_Text3:SetPoint("TOPLEFT", RemGank_Input,"TOPLEFT",20, -105)
RemGank_Input_Text3:SetFont(framefontn,framefonth)
RemGank_Input_Text3:SetTextColor(0.6,0.6,0.6,1)
RemGank_Input_Text3:SetText('Edit note')
	
-- EditBox1
RemGank_Input_eb1 = CreateFrame("EditBox", "eb1",RemGank_Input, "InputBoxTemplate")
RemGank_Input_eb1:SetPoint("TOPLEFT", RemGank_Input,"TOPLEFT",20, -120)
RemGank_Input_eb1:SetWidth(200)
RemGank_Input_eb1:SetHeight(13)
RemGank_Input_eb1:SetMaxLetters(25)
RemGank_Input_eb1:SetFont(framefontn,framefonth)
RemGank_Input_eb1:SetText('generic ganker')
RemGank_Input_eb1:SetAutoFocus(disable)

-- Save button
RemGank_Input_Save_Button = CreateFrame("button","RemGank_Input_Save_Button", RemGank_Input, "UIPanelButtonTemplate")
RemGank_Input_Save_Button:SetHeight(25)
RemGank_Input_Save_Button:SetWidth(50)
RemGank_Input_Save_Button:SetPoint("BOTTOMRIGHT", RemGank_Input, "BOTTOMRIGHT", -12, 12)
RemGank_Input_Save_Button:SetText("Yes")
RemGank_Input_Save_Button:SetScript("OnClick", 
		function()
			RemGank_Input:Hide()
			RemGank_Record_Player(RemGank_Input_Player:GetText(), RemGank_Input_eb1:GetText())
        end) 
	
-- Canc Button
RemGank_Input_Canc_Button = CreateFrame("button","RemGank_Input_Canc_Button", RemGank_Input, "UIPanelButtonTemplate")
RemGank_Input_Canc_Button:SetHeight(25)
RemGank_Input_Canc_Button:SetWidth(50)
RemGank_Input_Canc_Button:SetPoint("BOTTOMLEFT", RemGank_Input, "BOTTOMLEFT", 12, 12)
RemGank_Input_Canc_Button:SetText("No")
RemGank_Input_Canc_Button:SetScript("OnClick",  function() RemGank_Input:Hide() end) 
	

-- Frame to register the global vars at the ADDON_LOADED event.	
frame_al = CreateFrame("FRAME", "remgankframe_al")
frame_al:RegisterEvent("ADDON_LOADED")	
frame_al:SetScript("OnEvent", function(self, event, arg1, ...)

	if event == "ADDON_LOADED" and arg1 == "Remgank" then
		
		RemGank_DB_Dump()
		
		if REMGANK_ENABLE == 1 then 
			print(string_format("%s: enabled", prgname))
		else 
			print(string_format("%s: disabled. Type /remgank to configure", prgname))			
		end
		self:UnregisterEvent("ADDON_LOADED")
		
	end	
end)


-- Frame to register combat events to check
frame_cleu = CreateFrame("FRAME", "remgankframe_cleu")
frame_cleu:RegisterEvent("ZONE_CHANGED_NEW_AREA")
frame_cleu:RegisterEvent("PLAYER_ENTERING_WORLD")


frame_cleu:SetScript("OnEvent", function(self, event, ...)

	if event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_ENTERING_WORLD" then	
		
		local _, instanceType = IsInInstance()
		if REMGANK_ENABLE == 0 or instanceType == "arena" or instanceType == "pvp" or (instanceType == "none" and GetZonePVPInfo() == "combat") then
			-- REMGANK_ENABLE == 0 Monitoring is disabled
			-- arena is, obviously, an arena.
			-- pvp is a battleground.
			-- none with GetZonePVPInfo() == "combat" is an outdoor PvP zone like Wintergrasp.
			self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		else
			-- If not in a pvp zone, register CLEU:
			self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		end
		return
	end

	local timeStamp, event, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags, prefixParam1, prefixParam2, dummyparam, suffixParam1, suffixParam2 = ...
	
	
    -- proceed with CLEU handling
	if not playerGUID then
		playerGUID = UnitGUID("player")
	end

	-- Since these first two conditions apply no matter which event is
	-- firing, check them first.
	if destGUID == playerGUID and bit_band(tonumber(sourceGUID:sub(1,5)),0x00F) == 0 then
		if event == "SWING_DAMAGE" then
			-- Nest checks inside each other instead of using and, so
			-- that if any check matches (eg. it is this event, but not
			-- the right prefix) the code doesn't keep checking other
			-- things that obviously can never match.
			if prefixParam2 > 0 then
				print(string_format("%s: [%s] killed [%s] with %s Melee overkill %s", prgname, sourceName, destName, prefixParam1, prefixParam2))
				name = sourceName:lower()
				RemGank_Check_Player(name)
			end
		elseif event == "SPELL_DAMAGE" or event == "SPELL_PERIODIC_DAMAGE" or event == "RANGE_DAMAGE" then
			if suffixParam2 > 0 then
				print(string_format("%s: [%s] killed [%s] with %s damage of %s overkill %s", prgname, sourceName, destName, suffixParam1, GetSpellLink(prefixParam1), suffixParam2))
				name = sourceName:lower()
				RemGank_Check_Player(name)			
			end
		end
	end

end)
	
-- Frame to register UPDATE_MOUSEOVER_UNIT event.	
frame_umu = CreateFrame("FRAME", "remgankframe_umu")
frame_umu:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
frame_umu:SetScript("OnEvent", function (self, event, ...)
	if ((UnitIsPlayer("mouseover")) and (UnitIsEnemy("mouseover", "player"))) then
			name = UnitName("mouseover"):lower();
			if RemGankDB[name] then
				GameTooltip:AddLine(" ")
				GameTooltip:AddLine("- Your Note: " .. RemGankDB[name]["desc"])
				GameTooltip:AddLine("- Tot.kills: " .. RemGankDB[name]["nrkill"])
				GameTooltip:AddLine(" ")
				GameTooltip:AddLine("- Last time:")
				GameTooltip:AddLine(RemGankDB[name]["date"])
				GameTooltip:AddLine(RemGankDB[name]["lastloc"])				
				GameTooltipTextLeft1:SetTextColor(1, 0.5, 0.5)
				GameTooltip:SetBackdropColor(1, 0, 0)
				GameTooltip:Show()
			end
	end
 end);

-- Configuration Panel -------------------------------------------------------------------------------------

options = CreateFrame("Frame", ADDON.."Options", InterfaceOptionsFramePanelContainer)
options.name = GetAddOnMetadata(ADDON, "Title") or ADDON
InterfaceOptions_AddCategory(options)

title = options:CreateFontString("$parentTitle", "ARTWORK", "GameFontNormalLarge")
title:SetPoint("TOPLEFT", 16, -16)
title:SetText(options.name)
options.title = title

subtext = options:CreateFontString("$parentSubText", "ARTWORK", "GameFontHighlightSmall")
subtext:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
subtext:SetPoint("RIGHT", -32, 0)
subtext:SetHeight(32)
subtext:SetJustifyH("LEFT")
subtext:SetJustifyV("TOP")
subtext:SetText(GetAddOnMetadata(ADDON, "Notes"))
options.subtext = subtext

enable = CreateFrame("CheckButton", "$parentEnable", options, "InterfaceOptionsCheckButtonTemplate")
enable:SetPoint("TOPLEFT", subtext, "BOTTOMLEFT", -2, -8)
enable.Text:SetText(ENABLE)
enable.tooltipText = "Enable monitoring"
enable:SetChecked(REMGANK_ENABLE);
enable:SetScript("OnClick", function(self)
	
	if enable:GetChecked() then 
		REMGANK_ENABLE = 1
		print(string_format("%s: is enabled", prgname)) 
		frame_cleu:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	else 
		REMGANK_ENABLE = 0
		print(string_format("%s: is disabled",prgname))
		frame_cleu:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end	
	
end)
 
-- Database Text
titlelist = options:CreateFontString(nil, nil, "GameFontHighlight")
titlelist:SetPoint("TOPLEFT", enable, "BOTTOMLEFT", 2, -32)
titlelist:SetText("Players")

-- Add button
RemGank_Add_Player_Button = CreateFrame("button","RemGank_Add_Player_Button", options, "UIPanelButtonTemplate")
RemGank_Add_Player_Button:SetHeight(BUTTON_HEIGHT)
RemGank_Add_Player_Button:SetWidth(BUTTON_WIDTH/2)
RemGank_Add_Player_Button:SetPoint("TOPLEFT", titlelist, "BOTTOMLEFT", -2, -8)
RemGank_Add_Player_Button:SetText("Add")
RemGank_Add_Player_Button.tooltipText = "Add the selected player"
RemGank_Add_Player_Button:SetScript("OnClick", 
	function()
		print(string_format("%s: adding %s to the list", prgname, RemGank_Input_eb2:GetText()))
		RemGank_Record_Player(RemGank_Input_eb2:GetText())
		RemGank_DB_Dump()
		RemGank_ListFrame_Update(scrollFrame)
	end) 
	
-- Del Button
RemGank_Del_Player_Button = CreateFrame("button","RemGank_Del_Player_Button", options, "UIPanelButtonTemplate")
RemGank_Del_Player_Button:SetHeight(BUTTON_HEIGHT)
RemGank_Del_Player_Button:SetWidth(BUTTON_WIDTH/2)
RemGank_Del_Player_Button:SetPoint("TOPLEFT", titlelist, "BOTTOMLEFT", 48, -8)
RemGank_Del_Player_Button:SetText("Del")
RemGank_Del_Player_Button.tooltipText = "Delete the selected player"
RemGank_Del_Player_Button:SetScript("OnClick",  
	function()		
		RemGankDB[RemGank_Input_eb2:GetText()] = nil
		RemGank_DB_Dump()
		RemGank_ListFrame_Update(scrollFrame)
	end) 

-- Text Input
RemGank_Input_eb2 = CreateFrame("EditBox","eb2",options,"InputBoxTemplate")
RemGank_Input_eb2:SetMaxLetters(12)
RemGank_Input_eb2:SetFont(framefontn,framefonth)
RemGank_Input_eb2:SetAutoFocus(disable)
RemGank_Input_eb2:SetSize(BUTTON_WIDTH, BUTTON_HEIGHT)
RemGank_Input_eb2:SetPoint("TOPLEFT", titlelist, "BOTTOMLEFT", 104, -8)

-- ScrollFrame
scrollFrame = CreateFrame("ScrollFrame", "MyFirstNotReallyScrollFrame", options, "FauxScrollFrameTemplate")
scrollFrame:SetWidth(BUTTON_WIDTH)
scrollFrame:SetHeight(NUM_BUTTONS * BUTTON_HEIGHT)
scrollFrame:SetPoint("TOPLEFT", titlelist, "BOTTOMLEFT", 0, -40)
scrollFrame:EnableMouseWheel(enable)
scrollFrame:SetScript("OnVerticalScroll", function(self, offset)
	FauxScrollFrame_OnVerticalScroll(self, offset, BUTTON_HEIGHT, RemGank_ListFrame_Update(self))
end)

for i = 1, NUM_BUTTONS do
	local button = CreateFrame("Button", nil, scrollFrame:GetParent(), "UIPanelButtonTemplate")
	if i == 1 then
		button:SetPoint("TOP", scrollFrame)
	else
		button:SetPoint("TOP", buttons[i - 1], "BOTTOM")
	end
	button:SetNormalFontObject("GameFontNormal")
	button:SetSize(BUTTON_WIDTH, BUTTON_HEIGHT)
	button:SetText(RemGankLS[i])
	button:SetScript("OnClick", function(self) 
		RemGank_Input_eb2:SetText(button:GetText())
		info_name:SetText(button:GetText())
		info_desc:SetText(RemGankDB[button:GetText()]["desc"])
		info_loc_title:SetText("Last Kill:")
		info_date:SetText(RemGankDB[button:GetText()]["date"])
		info_loc:SetText(RemGankDB[button:GetText()]["lastloc"])
		info_kills_title:SetText("Tot.Kills:")
		info_kills:SetText(RemGankDB[button:GetText()]["nrkill"])
	end)
	buttons[i] = button
end


-- Info_name Text
info_name = options:CreateFontString(nil, nil, "GameFontNormalLarge")
info_name:SetPoint("TOPLEFT", titlelist, "BOTTOMLEFT", 160, -80)

-- Info_desc Text
info_desc = options:CreateFontString(nil, nil, "GameFontHighlightSmall")
info_desc:SetPoint("TOPLEFT", titlelist, "BOTTOMLEFT", 160, -100)

-- Info_loc_title Text
info_loc_title = options:CreateFontString(nil, nil, "GameFontNormal")
info_loc_title:SetPoint("TOPLEFT", titlelist, "BOTTOMLEFT", 160, -130)

-- Info_loc Text
info_loc = options:CreateFontString(nil, nil, "GameFontHighlightSmall")
info_loc:SetPoint("TOPLEFT", titlelist, "BOTTOMLEFT", 160, -145)

-- Info_date Text
info_date = options:CreateFontString(nil, nil, "GameFontHighlightSmall")
info_date:SetPoint("TOPLEFT", titlelist, "BOTTOMLEFT", 160, -160)

-- Info_kills_title Text
info_kills_title = options:CreateFontString(nil, nil, "GameFontNormal")
info_kills_title:SetPoint("TOPLEFT", titlelist, "BOTTOMLEFT", 160, -190)

-- Info_kills Text
info_kills = options:CreateFontString(nil, nil, "GameFontHighlightSmall")
info_kills:SetPoint("TOPLEFT", titlelist, "BOTTOMLEFT", 160, -205)

-- Tools Text
dbtoolstitle = options:CreateFontString(nil, nil, "GameFontHighlight")
dbtoolstitle:SetPoint("TOPLEFT", 16, -400)
dbtoolstitle:SetText("Database Tools")

-- Wipe Button
RemGank_Wip_Player_Button = CreateFrame("button","RemGank_Wip_Player_Button", options, "UIPanelButtonTemplate")
RemGank_Wip_Player_Button:SetHeight(BUTTON_HEIGHT)
RemGank_Wip_Player_Button:SetWidth(BUTTON_WIDTH/2)
RemGank_Wip_Player_Button:SetPoint("TOPLEFT", dbtoolstitle, "BOTTOMLEFT", -2, -8)
RemGank_Wip_Player_Button:SetText("Wipe")
RemGank_Wip_Player_Button.tooltipText = "BEWARE: Silently initialize your database of players"
RemGank_Wip_Player_Button:SetScript("OnClick",  
	function()
		print(prgname .. " Wipes current database")		
		RemGankLS = {}
		RemGankDB = {}
		RemGank_DB_Dump()
		RemGank_ListFrame_Update(scrollFrame)
	end)  

-- Reload Button
RemGank_Rld_Player_Button = CreateFrame("button","RemGank_Rld_Player_Button", options, "UIPanelButtonTemplate")
RemGank_Rld_Player_Button:SetHeight(BUTTON_HEIGHT)
RemGank_Rld_Player_Button:SetWidth(BUTTON_WIDTH/2)
RemGank_Rld_Player_Button:SetPoint("TOPLEFT", dbtoolstitle, "BOTTOMLEFT", 48, -8)
RemGank_Rld_Player_Button:SetText("Rld")
RemGank_Rld_Player_Button.tooltipText = "Rebuild list from Database"
RemGank_Rld_Player_Button:SetScript("OnClick",  
	function()
		RemGank_DB_Dump()
		RemGank_ListFrame_Update(scrollFrame)
	end)  

	
	
	
	
	
function options.refresh()
	enable:SetChecked(REMGANK_ENABLE)
	RemGank_ListFrame_Update(scrollFrame)
end

if LibStub and LibStub("LibAboutPanel", true) then
	options.about = LibStub("LibAboutPanel").new(options.name, ADDON)
end
 -- END Configuration Panel ----------------------------------------------------------------------------------
 

SLASH_REMGANK1 = "/remgank"
SlashCmdList["REMGANK"] = function() 
	InterfaceOptionsFrame_OpenToCategory(options)
end


